@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2024 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,20 +18,20 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_RENDER_SW
+#ifdef SDL_VIDEO_RENDER_SW
 
 #include "SDL_draw.h"
 #include "SDL_drawline.h"
 #include "SDL_drawpoint.h"
 
 static void SDL_DrawLine1(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint32 color,
-                          SDL_bool draw_end)
+                          bool draw_end)
 {
     if (y1 == y2) {
         int length;
-        int pitch = (dst->pitch / dst->format->BytesPerPixel);
+        int pitch = (dst->pitch / dst->fmt->bytes_per_pixel);
         Uint8 *pixel;
         if (x1 <= x2) {
             pixel = (Uint8 *)dst->pixels + y1 * pitch + x1;
@@ -54,7 +54,7 @@ static void SDL_DrawLine1(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint
 }
 
 static void SDL_DrawLine2(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint32 color,
-                          SDL_bool draw_end)
+                          bool draw_end)
 {
     if (y1 == y2) {
         HLINE(Uint16, DRAW_FASTSETPIXEL2, draw_end);
@@ -64,8 +64,8 @@ static void SDL_DrawLine2(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint
         DLINE(Uint16, DRAW_FASTSETPIXEL2, draw_end);
     } else {
         Uint8 _r, _g, _b, _a;
-        const SDL_PixelFormat *fmt = dst->format;
-        SDL_GetRGBA(color, fmt, &_r, &_g, &_b, &_a);
+        const SDL_PixelFormatDetails *fmt = dst->fmt;
+        SDL_GetRGBA(color, fmt, dst->palette, &_r, &_g, &_b, &_a);
         if (fmt->Rmask == 0x7C00) {
             AALINE(x1, y1, x2, y2,
                    DRAW_FASTSETPIXELXY2, DRAW_SETPIXELXY_BLEND_RGB555,
@@ -83,7 +83,7 @@ static void SDL_DrawLine2(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint
 }
 
 static void SDL_DrawLine4(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint32 color,
-                          SDL_bool draw_end)
+                          bool draw_end)
 {
     if (y1 == y2) {
         HLINE(Uint32, DRAW_FASTSETPIXEL4, draw_end);
@@ -93,12 +93,12 @@ static void SDL_DrawLine4(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint
         DLINE(Uint32, DRAW_FASTSETPIXEL4, draw_end);
     } else {
         Uint8 _r, _g, _b, _a;
-        const SDL_PixelFormat *fmt = dst->format;
-        SDL_GetRGBA(color, fmt, &_r, &_g, &_b, &_a);
+        const SDL_PixelFormatDetails *fmt = dst->fmt;
+        SDL_GetRGBA(color, fmt, dst->palette, &_r, &_g, &_b, &_a);
         if (fmt->Rmask == 0x00FF0000) {
             if (!fmt->Amask) {
                 AALINE(x1, y1, x2, y2,
-                       DRAW_FASTSETPIXELXY4, DRAW_SETPIXELXY_BLEND_RGB888,
+                       DRAW_FASTSETPIXELXY4, DRAW_SETPIXELXY_BLEND_XRGB8888,
                        draw_end);
             } else {
                 AALINE(x1, y1, x2, y2,
@@ -115,13 +115,13 @@ static void SDL_DrawLine4(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint
 
 typedef void (*DrawLineFunc)(SDL_Surface *dst,
                              int x1, int y1, int x2, int y2,
-                             Uint32 color, SDL_bool draw_end);
+                             Uint32 color, bool draw_end);
 
-static DrawLineFunc SDL_CalculateDrawLineFunc(const SDL_PixelFormat *fmt)
+static DrawLineFunc SDL_CalculateDrawLineFunc(const SDL_PixelFormatDetails *fmt)
 {
-    switch (fmt->BytesPerPixel) {
+    switch (fmt->bytes_per_pixel) {
     case 1:
-        if (fmt->BitsPerPixel < 8) {
+        if (fmt->bits_per_pixel < 8) {
             break;
         }
         return SDL_DrawLine1;
@@ -133,43 +133,42 @@ static DrawLineFunc SDL_CalculateDrawLineFunc(const SDL_PixelFormat *fmt)
     return NULL;
 }
 
-int SDL_DrawLine(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint32 color)
+bool SDL_DrawLine(SDL_Surface *dst, int x1, int y1, int x2, int y2, Uint32 color)
 {
     DrawLineFunc func;
 
-    if (!dst) {
+    if (!SDL_SurfaceValid(dst)) {
         return SDL_InvalidParamError("SDL_DrawLine(): dst");
     }
 
-    func = SDL_CalculateDrawLineFunc(dst->format);
+    func = SDL_CalculateDrawLineFunc(dst->fmt);
     if (!func) {
         return SDL_SetError("SDL_DrawLine(): Unsupported surface format");
     }
 
-    /* Perform clipping */
-    /* FIXME: We don't actually want to clip, as it may change line slope */
-    if (!SDL_IntersectRectAndLine(&dst->clip_rect, &x1, &y1, &x2, &y2)) {
-        return 0;
+    // Perform clipping
+    // FIXME: We don't actually want to clip, as it may change line slope
+    if (!SDL_GetRectAndLineIntersection(&dst->clip_rect, &x1, &y1, &x2, &y2)) {
+        return true;
     }
 
-    func(dst, x1, y1, x2, y2, color, SDL_TRUE);
-    return 0;
+    func(dst, x1, y1, x2, y2, color, true);
+    return true;
 }
 
-int SDL_DrawLines(SDL_Surface *dst, const SDL_Point *points, int count,
-                  Uint32 color)
+bool SDL_DrawLines(SDL_Surface *dst, const SDL_Point *points, int count, Uint32 color)
 {
     int i;
     int x1, y1;
     int x2, y2;
-    SDL_bool draw_end;
+    bool draw_end;
     DrawLineFunc func;
 
-    if (!dst) {
+    if (!SDL_SurfaceValid(dst)) {
         return SDL_InvalidParamError("SDL_DrawLines(): dst");
     }
 
-    func = SDL_CalculateDrawLineFunc(dst->format);
+    func = SDL_CalculateDrawLineFunc(dst->fmt);
     if (!func) {
         return SDL_SetError("SDL_DrawLines(): Unsupported surface format");
     }
@@ -180,13 +179,13 @@ int SDL_DrawLines(SDL_Surface *dst, const SDL_Point *points, int count,
         x2 = points[i].x;
         y2 = points[i].y;
 
-        /* Perform clipping */
-        /* FIXME: We don't actually want to clip, as it may change line slope */
-        if (!SDL_IntersectRectAndLine(&dst->clip_rect, &x1, &y1, &x2, &y2)) {
+        // Perform clipping
+        // FIXME: We don't actually want to clip, as it may change line slope
+        if (!SDL_GetRectAndLineIntersection(&dst->clip_rect, &x1, &y1, &x2, &y2)) {
             continue;
         }
 
-        /* Draw the end if the whole line is a single point or it was clipped */
+        // Draw the end if the whole line is a single point or it was clipped
         draw_end = ((x1 == x2) && (y1 == y2)) || (x2 != points[i].x || y2 != points[i].y);
 
         func(dst, x1, y1, x2, y2, color, draw_end);
@@ -194,9 +193,7 @@ int SDL_DrawLines(SDL_Surface *dst, const SDL_Point *points, int count,
     if (points[0].x != points[count - 1].x || points[0].y != points[count - 1].y) {
         SDL_DrawPoint(dst, points[count - 1].x, points[count - 1].y, color);
     }
-    return 0;
+    return true;
 }
 
-#endif /* SDL_VIDEO_RENDER_SW */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_RENDER_SW
