@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,15 +18,12 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_VIDEO_DRIVER_UIKIT
+#ifdef SDL_VIDEO_DRIVER_UIKIT
 
 #import <UIKit/UIKit.h>
 
-#include "SDL_video.h"
-#include "SDL_mouse.h"
-#include "SDL_hints.h"
 #include "../SDL_sysvideo.h"
 #include "../SDL_pixels_c.h"
 #include "../../events/SDL_events_c.h"
@@ -39,23 +36,27 @@
 #include "SDL_uikitclipboard.h"
 #include "SDL_uikitvulkan.h"
 #include "SDL_uikitmetalview.h"
+#include "SDL_uikitmessagebox.h"
+#include "SDL_uikitpen.h"
 
 #define UIKITVID_DRIVER_NAME "uikit"
 
-@implementation SDL_VideoData
+@implementation SDL_UIKitVideoData
 
 @end
 
-/* Initialization/Query functions */
-static int UIKit_VideoInit(_THIS);
-static void UIKit_VideoQuit(_THIS);
+// Initialization/Query functions
+static bool UIKit_VideoInit(SDL_VideoDevice *_this);
+static void UIKit_VideoQuit(SDL_VideoDevice *_this);
 
-/* DUMMY driver bootstrap functions */
+// DUMMY driver bootstrap functions
 
-static void UIKit_DeleteDevice(SDL_VideoDevice * device)
+static void UIKit_DeleteDevice(SDL_VideoDevice *device)
 {
     @autoreleasepool {
-        CFRelease(device->driverdata);
+        if (device->internal){
+            CFRelease(device->internal);
+        }
         SDL_free(device);
     }
 }
@@ -64,21 +65,20 @@ static SDL_VideoDevice *UIKit_CreateDevice(void)
 {
     @autoreleasepool {
         SDL_VideoDevice *device;
-        SDL_VideoData *data;
+        SDL_UIKitVideoData *data;
 
-        /* Initialize all variables that we clean on shutdown */
-        device = (SDL_VideoDevice *) SDL_calloc(1, sizeof(SDL_VideoDevice));
-        if (device) {
-            data = [SDL_VideoData new];
-        } else {
-            SDL_free(device);
-            SDL_OutOfMemory();
-            return (0);
+        // Initialize all variables that we clean on shutdown
+        device = (SDL_VideoDevice *)SDL_calloc(1, sizeof(SDL_VideoDevice));
+        if (!device) {
+            return NULL;
         }
 
-        device->driverdata = (void *) CFBridgingRetain(data);
+        data = [SDL_UIKitVideoData new];
 
-        /* Set the function pointers */
+        device->internal = (SDL_VideoData *)CFBridgingRetain(data);
+        device->system_theme = UIKit_GetSystemTheme();
+
+        // Set the function pointers
         device->VideoInit = UIKit_VideoInit;
         device->VideoQuit = UIKit_VideoQuit;
         device->GetDisplayModes = UIKit_GetDisplayModes;
@@ -92,53 +92,50 @@ static SDL_VideoDevice *UIKit_CreateDevice(void)
         device->RaiseWindow = UIKit_RaiseWindow;
         device->SetWindowBordered = UIKit_SetWindowBordered;
         device->SetWindowFullscreen = UIKit_SetWindowFullscreen;
-        device->SetWindowMouseGrab = UIKit_SetWindowMouseGrab;
         device->DestroyWindow = UIKit_DestroyWindow;
-        device->GetWindowWMInfo = UIKit_GetWindowWMInfo;
         device->GetDisplayUsableBounds = UIKit_GetDisplayUsableBounds;
-        device->GetDisplayDPI = UIKit_GetDisplayDPI;
         device->GetWindowSizeInPixels = UIKit_GetWindowSizeInPixels;
 
-#if SDL_IPHONE_KEYBOARD
+#ifdef SDL_IPHONE_KEYBOARD
         device->HasScreenKeyboardSupport = UIKit_HasScreenKeyboardSupport;
-        device->ShowScreenKeyboard = UIKit_ShowScreenKeyboard;
-        device->HideScreenKeyboard = UIKit_HideScreenKeyboard;
+        device->StartTextInput = UIKit_StartTextInput;
+        device->StopTextInput = UIKit_StopTextInput;
+        device->SetTextInputProperties = UIKit_SetTextInputProperties;
         device->IsScreenKeyboardShown = UIKit_IsScreenKeyboardShown;
-        device->SetTextInputRect = UIKit_SetTextInputRect;
+        device->UpdateTextInputArea = UIKit_UpdateTextInputArea;
 #endif
 
         device->SetClipboardText = UIKit_SetClipboardText;
         device->GetClipboardText = UIKit_GetClipboardText;
         device->HasClipboardText = UIKit_HasClipboardText;
 
-        /* OpenGL (ES) functions */
-#if SDL_VIDEO_OPENGL_ES || SDL_VIDEO_OPENGL_ES2
-        device->GL_MakeCurrent      = UIKit_GL_MakeCurrent;
-        device->GL_GetDrawableSize  = UIKit_GL_GetDrawableSize;
-        device->GL_SwapWindow       = UIKit_GL_SwapWindow;
-        device->GL_CreateContext    = UIKit_GL_CreateContext;
-        device->GL_DeleteContext    = UIKit_GL_DeleteContext;
-        device->GL_GetProcAddress   = UIKit_GL_GetProcAddress;
-        device->GL_LoadLibrary      = UIKit_GL_LoadLibrary;
+        // OpenGL (ES) functions
+#if defined(SDL_VIDEO_OPENGL_ES) || defined(SDL_VIDEO_OPENGL_ES2)
+        device->GL_MakeCurrent = UIKit_GL_MakeCurrent;
+        device->GL_SwapWindow = UIKit_GL_SwapWindow;
+        device->GL_CreateContext = UIKit_GL_CreateContext;
+        device->GL_DestroyContext = UIKit_GL_DestroyContext;
+        device->GL_GetProcAddress = UIKit_GL_GetProcAddress;
+        device->GL_LoadLibrary = UIKit_GL_LoadLibrary;
 #endif
         device->free = UIKit_DeleteDevice;
 
-#if SDL_VIDEO_VULKAN
+#ifdef SDL_VIDEO_VULKAN
         device->Vulkan_LoadLibrary = UIKit_Vulkan_LoadLibrary;
         device->Vulkan_UnloadLibrary = UIKit_Vulkan_UnloadLibrary;
-        device->Vulkan_GetInstanceExtensions
-                                     = UIKit_Vulkan_GetInstanceExtensions;
+        device->Vulkan_GetInstanceExtensions = UIKit_Vulkan_GetInstanceExtensions;
         device->Vulkan_CreateSurface = UIKit_Vulkan_CreateSurface;
-        device->Vulkan_GetDrawableSize = UIKit_Vulkan_GetDrawableSize;
+        device->Vulkan_DestroySurface = UIKit_Vulkan_DestroySurface;
 #endif
 
-#if SDL_VIDEO_METAL
+#ifdef SDL_VIDEO_METAL
         device->Metal_CreateView = UIKit_Metal_CreateView;
         device->Metal_DestroyView = UIKit_Metal_DestroyView;
         device->Metal_GetLayer = UIKit_Metal_GetLayer;
-        device->Metal_GetDrawableSize = UIKit_Metal_GetDrawableSize;
 #endif
 
+        device->device_caps = VIDEO_DEVICE_CAPS_SENDS_FULLSCREEN_DIMENSIONS;
+
         device->gl_config.accelerated = 1;
 
         return device;
@@ -147,54 +144,79 @@ static SDL_VideoDevice *UIKit_CreateDevice(void)
 
 VideoBootStrap UIKIT_bootstrap = {
     UIKITVID_DRIVER_NAME, "SDL UIKit video driver",
-    UIKit_CreateDevice
+    UIKit_CreateDevice,
+    UIKit_ShowMessageBox,
+    false
 };
 
-
-int UIKit_VideoInit(_THIS)
+static bool UIKit_VideoInit(SDL_VideoDevice *_this)
 {
     _this->gl_config.driver_loaded = 1;
 
-    if (UIKit_InitModes(_this) < 0) {
-        return -1;
+    if (!UIKit_InitModes(_this)) {
+        return false;
     }
 
     SDL_InitGCKeyboard();
     SDL_InitGCMouse();
 
-    return 0;
+    UIKit_InitClipboard(_this);
+
+    return true;
 }
 
-void UIKit_VideoQuit(_THIS)
+static void UIKit_VideoQuit(SDL_VideoDevice *_this)
 {
+    UIKit_QuitClipboard(_this);
+
     SDL_QuitGCKeyboard();
     SDL_QuitGCMouse();
+    UIKit_QuitPen(_this);
 
     UIKit_QuitModes(_this);
 }
 
-void UIKit_SuspendScreenSaver(_THIS)
+bool UIKit_SuspendScreenSaver(SDL_VideoDevice *_this)
 {
     @autoreleasepool {
-        /* Ignore ScreenSaver API calls if the idle timer hint has been set. */
-        /* FIXME: The idle timer hint should be deprecated for SDL 2.1. */
-        if (!SDL_GetHintBoolean(SDL_HINT_IDLE_TIMER_DISABLED, SDL_FALSE)) {
-            UIApplication *app = [UIApplication sharedApplication];
+        UIApplication *app = [UIApplication sharedApplication];
 
-            /* Prevent the display from dimming and going to sleep. */
-            app.idleTimerDisabled = (_this->suspend_screensaver != SDL_FALSE);
-        }
+        // Prevent the display from dimming and going to sleep.
+        app.idleTimerDisabled = (_this->suspend_screensaver != false);
     }
+    return true;
 }
 
-SDL_bool UIKit_IsSystemVersionAtLeast(double version)
+bool UIKit_IsSystemVersionAtLeast(double version)
 {
     return [[UIDevice currentDevice].systemVersion doubleValue] >= version;
 }
 
+SDL_SystemTheme UIKit_GetSystemTheme(void)
+{
+#ifndef SDL_PLATFORM_VISIONOS
+    if (@available(iOS 12.0, tvOS 10.0, *)) {
+        switch ([UIScreen mainScreen].traitCollection.userInterfaceStyle) {
+        case UIUserInterfaceStyleDark:
+            return SDL_SYSTEM_THEME_DARK;
+        case UIUserInterfaceStyleLight:
+            return SDL_SYSTEM_THEME_LIGHT;
+        default:
+            break;
+        }
+    }
+#endif
+    return SDL_SYSTEM_THEME_UNKNOWN;
+}
+
+#ifdef SDL_PLATFORM_VISIONOS
+CGRect UIKit_ComputeViewFrame(SDL_Window *window){
+    return CGRectMake(window->x, window->y, window->w, window->h);
+}
+#else
 CGRect UIKit_ComputeViewFrame(SDL_Window *window, UIScreen *screen)
 {
-    SDL_WindowData *data = (__bridge SDL_WindowData *) window->driverdata;
+    SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)window->internal;
     CGRect frame = screen.bounds;
 
     /* Use the UIWindow bounds instead of the UIScreen bounds, when possible.
@@ -204,7 +226,7 @@ CGRect UIKit_ComputeViewFrame(SDL_Window *window, UIScreen *screen)
         frame = data.uiwindow.bounds;
     }
 
-#if !TARGET_OS_TV
+#ifndef SDL_PLATFORM_TVOS
     /* iOS 10 seems to have a bug where, in certain conditions, putting the
      * device to sleep with the a landscape-only app open, re-orienting the
      * device to portrait, and turning it back on will result in the screen
@@ -230,30 +252,27 @@ CGRect UIKit_ComputeViewFrame(SDL_Window *window, UIScreen *screen)
     return frame;
 }
 
-void UIKit_ForceUpdateHomeIndicator()
+#endif
+
+void UIKit_ForceUpdateHomeIndicator(void)
 {
-#if !TARGET_OS_TV
-    /* Force the main SDL window to re-evaluate home indicator state */
-    SDL_Window *focus = SDL_GetFocusWindow();
+#ifndef SDL_PLATFORM_TVOS
+    // Force the main SDL window to re-evaluate home indicator state
+    SDL_Window *focus = SDL_GetKeyboardFocus();
     if (focus) {
-        SDL_WindowData *data = (__bridge SDL_WindowData *) focus->driverdata;
+        SDL_UIKitWindowData *data = (__bridge SDL_UIKitWindowData *)focus->internal;
         if (data != nil) {
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wunguarded-availability-new"
-            if ([data.viewcontroller respondsToSelector:@selector(setNeedsUpdateOfHomeIndicatorAutoHidden)]) {
-                [data.viewcontroller performSelectorOnMainThread:@selector(setNeedsUpdateOfHomeIndicatorAutoHidden) withObject:nil waitUntilDone:NO];
-                [data.viewcontroller performSelectorOnMainThread:@selector(setNeedsUpdateOfScreenEdgesDeferringSystemGestures) withObject:nil waitUntilDone:NO];
-            }
-#pragma clang diagnostic pop
+            [data.viewcontroller performSelectorOnMainThread:@selector(setNeedsUpdateOfHomeIndicatorAutoHidden) withObject:nil waitUntilDone:NO];
+            [data.viewcontroller performSelectorOnMainThread:@selector(setNeedsUpdateOfScreenEdgesDeferringSystemGestures) withObject:nil waitUntilDone:NO];
         }
     }
-#endif /* !TARGET_OS_TV */
+#endif // !SDL_PLATFORM_TVOS
 }
 
 /*
  * iOS log support.
  *
- * This doesn't really have aything to do with the interfaces of the SDL video
+ * This doesn't really have anything to do with the interfaces of the SDL video
  *  subsystem, but we need to stuff this into an Objective-C source code file.
  *
  * NOTE: This is copypasted from src/video/cocoa/SDL_cocoavideo.m! Thus, if
@@ -261,32 +280,35 @@ void UIKit_ForceUpdateHomeIndicator()
  *  identical!
  */
 
-#if !defined(SDL_VIDEO_DRIVER_COCOA)
+#ifndef SDL_VIDEO_DRIVER_COCOA
 void SDL_NSLog(const char *prefix, const char *text)
 {
     @autoreleasepool {
         NSString *nsText = [NSString stringWithUTF8String:text];
-        if (prefix) {
+        if (prefix && *prefix) {
             NSString *nsPrefix = [NSString stringWithUTF8String:prefix];
-            NSLog(@"%@: %@", nsPrefix, nsText);
+            NSLog(@"%@%@", nsPrefix, nsText);
         } else {
             NSLog(@"%@", nsText);
         }
     }
 }
-#endif /* SDL_VIDEO_DRIVER_COCOA */
+#endif // SDL_VIDEO_DRIVER_COCOA
 
 /*
- * iOS Tablet detection
+ * iOS Tablet, etc, detection
  *
- * This doesn't really have aything to do with the interfaces of the SDL video
+ * This doesn't really have anything to do with the interfaces of the SDL video
  * subsystem, but we need to stuff this into an Objective-C source code file.
  */
-SDL_bool SDL_IsIPad(void)
+bool SDL_IsIPad(void)
 {
     return ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad);
 }
 
-#endif /* SDL_VIDEO_DRIVER_UIKIT */
+bool SDL_IsAppleTV(void)
+{
+    return ([UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomTV);
+}
 
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_VIDEO_DRIVER_UIKIT
